"""Test backendconfig models."""

import pytest
from pydantic import ValidationError

from quantinuum_schemas.models.backend_config import (
    AerConfig,
    SeleneConfig,
    QuantinuumCompilerOptions,
    SelenePlusConfig,
)
from quantinuum_schemas.models.emulator_config import (
    ClassicalReplaySimulator,
    CoinflipSimulator,
    DepolarizingErrorModel,
    HeliosRuntime,
    MatrixProductStateSimulator,
    NoErrorModel,
    QSystemErrorModel,
    SimpleRuntime,
    StabilizerSimulator,
    StatevectorSimulator,
)


def test_instantiation() -> None:
    """Test instantiation of AerConfig"""
    aer_config = AerConfig()
    assert isinstance(aer_config, AerConfig)


def test_valid_quantinuum_compiler_options() -> None:
    """Test to ensure that all expected arguments can be accepted by the compiler options class"""
    dict_of_options = {
        "expect_threshold": 0.5,
        "DD_threshold_times": [0.1, 0.2, 0.3],
        "CF": "non linear",
        "test_cz": False,
        "max_planning": 601,
    }

    QuantinuumCompilerOptions(**dict_of_options)


def test_handling_invalid_option() -> None:
    """Expect an assert error raised when passing a bad compiler option"""
    dict_of_options = {
        "DD_threshold_times": [0.1, 3, 0.3],
    }

    with pytest.raises(ValidationError):
        QuantinuumCompilerOptions(**dict_of_options)


@pytest.mark.parametrize("runtime_class", [SimpleRuntime])
@pytest.mark.parametrize("error_model_class", [NoErrorModel, DepolarizingErrorModel])
@pytest.mark.parametrize(
    "simulator_class",
    [
        StabilizerSimulator,
        StatevectorSimulator,
        CoinflipSimulator,
        ClassicalReplaySimulator,
    ],
)
def test_selene_config_roundtrip(
    runtime_class: type,
    error_model_class: type,
    simulator_class: type,
) -> None:
    """Test roundtrip of SeleneConfig, importantly the ability to discriminate the
    error model and the runtime."""

    config = SeleneConfig(
        simulator=simulator_class(),
        runtime=runtime_class(),
        error_model=error_model_class(),
        n_qubits=4,
    )

    reloaded_config = SeleneConfig.model_validate_json(config.model_dump_json())

    assert config == reloaded_config
    assert config.runtime == reloaded_config.runtime
    assert config.error_model == reloaded_config.error_model


@pytest.mark.parametrize("runtime_class", [SimpleRuntime, HeliosRuntime])
@pytest.mark.parametrize(
    "error_model_class", [NoErrorModel, DepolarizingErrorModel, QSystemErrorModel]
)
@pytest.mark.parametrize(
    "simulator_class",
    [
        StabilizerSimulator,
        StatevectorSimulator,
        CoinflipSimulator,
        MatrixProductStateSimulator,
        ClassicalReplaySimulator,
    ],
)
def test_selene_plus_config_roundtrip(
    runtime_class: type,
    simulator_class: type,
    error_model_class: type,
) -> None:
    """Test roundtrip of SelenePlusConfig, importantly the ability to discriminate the
    error model and the runtime."""

    config = SelenePlusConfig(
        runtime=runtime_class(),
        simulator=simulator_class(),
        error_model=error_model_class(),
        n_qubits=4,
    )

    reloaded_config = SelenePlusConfig.model_validate_json(config.model_dump_json())

    assert config == reloaded_config
    assert config.runtime == reloaded_config.runtime
    assert config.error_model == reloaded_config.error_model



"""Tests for result types."""

import pytest
from pydantic import BaseModel
from pydantic.error_wrappers import ValidationError

from quantinuum_schemas.models.result import QSysResult


def test_result_serialisation() -> None:
    """Test serialisation of result types."""

    class TestResultModel(BaseModel):
        """Test result model to check pydantic handling."""

        result: QSysResult

    test_result_1 = [
        [
            ["test_key_1", 1],
            ["test_key_2", [1, 2, 3]],
        ],
        [
            ["test_key_3", 1.0],
            ["test_key_4", [1.121341453, 2.7878993, 334.0000123]],
        ],
        [
            ["test_key_5", 1],
            ["test_key_6", [True, 0, False, 1]],
        ],
    ]

    test_result_model_instance = TestResultModel(result=test_result_1)  # type: ignore[arg-type]

    assert test_result_model_instance == TestResultModel.model_validate_json(
        test_result_model_instance.model_dump_json()
    )

    test_result_2 = [
        [
            ["test_key_1", "hello"],
            ["test_key_2", ["hi", "hrllo"]],
        ],
        [
            ["test_key_3", [[1.0, 2.0], [True, False]]],
        ],
    ]

    with pytest.raises(ValidationError):
        TestResultModel(result=test_result_2)  # type: ignore[arg-type]



"""Test hypertket config model."""

from quantinuum_schemas.models.hypertket_config import (
    DualStrat,
    HyperTketConfig,
    QubitReuseConfig,
)


def test_instantiate() -> None:
    """Test instantiation of HyperTketConfig."""
    HyperTketConfig()


def test_enum_serialization() -> None:
    """Test serialization of DualStrat."""
    config = HyperTketConfig(
        qubit_reuse_config=QubitReuseConfig(dual_circuit_strategy=DualStrat.AUTO)
    )

    serialised = config.model_dump()

    HyperTketConfig.model_validate(serialised)
